@@ -119,9 +119,9 @@ parser_groups = (
         (("", "--extra-packages",), dict(dest="extra_packages",
                                          help=("extra packages to include, "
                                                "comma-separated. Default is "
-                                               "'addon-kit'."),
+                                               "'addon-sdk'."),
                                          metavar=None,
-                                         default="addon-kit",
+                                         default="addon-sdk",
                                          cmds=['run', 'xpi', 'test', 'testex',
                                                'testpkgs', 'testall',
                                                'testcfx'])),
@@ -194,6 +194,11 @@ parser_groups = (
                                   metavar=None,
                                   default=False,
                                   cmds=['test', 'testex', 'testpkgs'])),
+        (("", "--override-version",), dict(dest="override_version",
+                                  help="Pass in a version string to use in generated docs",
+                                  metavar=None,
+                                  default=False,
+                                  cmds=['sdocs'])),
         ]
      ),
 
@@ -216,7 +221,7 @@ parser_groups = (
                                                 "containing test runner "
                                                 "program (default is "
                                                 "test-harness)"),
-                                          default="test-harness",
+                                          default="addon-sdk",
                                           cmds=['test', 'testex', 'testpkgs',
                                                 'testall'])),
         # --keydir was removed in 1.0b5, but we keep it around in the options
@@ -323,11 +328,12 @@ def parse_args(arguments, global_options, usage, version, parser_groups,
 def test_all(env_root, defaults):
     fail = False
 
-    print >>sys.stderr, "Testing cfx..."
-    sys.stderr.flush()
-    result = test_cfx(env_root, defaults['verbose'])
-    if result.failures or result.errors:
-        fail = True
+    if not defaults['filter']:
+        print >>sys.stderr, "Testing cfx..."
+        sys.stderr.flush()
+        result = test_cfx(env_root, defaults['verbose'])
+        if result.failures or result.errors:
+            fail = True
 
     if not fail or not defaults.get("stopOnError"):
         print >>sys.stderr, "Testing all examples..."
@@ -338,6 +344,15 @@ def test_all(env_root, defaults):
         except SystemExit, e:
             fail = (e.code != 0) or fail
 
+    if not fail or not defaults.get("stopOnError"):
+        print >>sys.stderr, "Testing all unit-test addons..."
+        sys.stderr.flush()
+
+        try:
+            test_all_testaddons(env_root, defaults)
+        except SystemExit, e:
+            fail = (e.code != 0) or fail
+
     if not fail or not defaults.get("stopOnError"):
         print >>sys.stderr, "Testing all packages..."
         sys.stderr.flush()
@@ -365,6 +380,30 @@ def test_cfx(env_root, verbose):
     sys.stdout.flush(); sys.stderr.flush()
     return retval
 
+def test_all_testaddons(env_root, defaults):
+    addons_dir = os.path.join(env_root, "test", "addons")
+    addons = [dirname for dirname in os.listdir(addons_dir)
+                if os.path.isdir(os.path.join(addons_dir, dirname))]
+    addons.sort()
+    fail = False
+    for dirname in addons:
+        print >>sys.stderr, "Testing %s..." % dirname
+        sys.stderr.flush()
+        try:
+            run(arguments=["run",
+                           "--pkgdir",
+                           os.path.join(addons_dir, dirname)],
+                defaults=defaults,
+                env_root=env_root)
+        except SystemExit, e:
+            fail = (e.code != 0) or fail
+        if fail and defaults.get("stopOnError"):
+            break
+
+    if fail:
+        print >>sys.stderr, "Some test addons tests were unsuccessful."
+        sys.exit(-1)
+
 def test_all_examples(env_root, defaults):
     examples_dir = os.path.join(env_root, "examples")
     examples = [dirname for dirname in os.listdir(examples_dir)
@@ -391,8 +430,12 @@ def test_all_examples(env_root, defaults):
 
 def test_all_packages(env_root, defaults):
     packages_dir = os.path.join(env_root, "packages")
-    packages = [dirname for dirname in os.listdir(packages_dir)
-                if os.path.isdir(os.path.join(packages_dir, dirname))]
+    if os.path.isdir(packages_dir):
+      packages = [dirname for dirname in os.listdir(packages_dir)
+                  if os.path.isdir(os.path.join(packages_dir, dirname))]
+    else:
+      packages = []
+    packages.append(env_root)
     packages.sort()
     print >>sys.stderr, "Testing all available packages: %s." % (", ".join(packages))
     sys.stderr.flush()
@@ -440,13 +483,20 @@ def get_config_args(name, env_root):
     return config
 
 def initializer(env_root, args, out=sys.stdout, err=sys.stderr):
-    from templates import MAIN_JS, PACKAGE_JSON, README_DOC, MAIN_JS_DOC, TEST_MAIN_JS
+    from templates import PACKAGE_JSON, TEST_MAIN_JS
     path = os.getcwd()
     addon = os.path.basename(path)
-    # if more than one argument
-    if len(args) > 1:
+    # if more than two arguments
+    if len(args) > 2:
         print >>err, 'Too many arguments.'
         return 1
+    if len(args) == 2:
+        path = os.path.join(path,args[1])
+        try:
+            os.mkdir(path)
+            print >>out, '*', args[1], 'package directory created'
+        except OSError:
+            print >>out, '*', args[1], 'already exists, testing if directory is empty'
     # avoid clobbering existing files, but we tolerate things like .git
     existing = [fn for fn in os.listdir(path) if not fn.startswith(".")]
     if existing:
@@ -455,19 +505,23 @@ def initializer(env_root, args, out=sys.stdout, err=sys.stderr):
     for d in ['lib','data','test','doc']:
         os.mkdir(os.path.join(path,d))
         print >>out, '*', d, 'directory created'
-    open('README.md','w').write(README_DOC % {'name':addon})
+    open(os.path.join(path,'README.md'),'w').write('')
     print >>out, '* README.md written'
-    open('package.json','w').write(PACKAGE_JSON % {'name':addon.lower(),
+    open(os.path.join(path,'package.json'),'w').write(PACKAGE_JSON % {'name':addon.lower(),
                                                    'fullName':addon })
     print >>out, '* package.json written'
     open(os.path.join(path,'test','test-main.js'),'w').write(TEST_MAIN_JS)
     print >>out, '* test/test-main.js written'
-    open(os.path.join(path,'lib','main.js'),'w').write(MAIN_JS)
+    open(os.path.join(path,'lib','main.js'),'w').write('')
     print >>out, '* lib/main.js written'
-    open(os.path.join(path,'doc','main.md'),'w').write(MAIN_JS_DOC)
+    open(os.path.join(path,'doc','main.md'),'w').write('')
     print >>out, '* doc/main.md written'
-    print >>out, '\nYour sample add-on is now ready.'
-    print >>out, 'Do "cfx test" to test it and "cfx run" to try it.  Have fun!'
+    if len(args) == 1:
+        print >>out, '\nYour sample add-on is now ready.'
+        print >>out, 'Do "cfx test" to test it and "cfx run" to try it.  Have fun!'
+    else:
+        print >>out, '\nYour sample add-on is now ready in the \'' + args[1] +  '\' directory.'
+        print >>out, 'Change to that directory, then do "cfx test" to test it, \nand "cfx run" to try it.  Have fun!' 
     return 0
 
 def buildJID(target_cfg):
@@ -510,6 +564,9 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
     if command == "testpkgs":
         test_all_packages(env_root, defaults=options.__dict__)
         return
+    elif command == "testaddons":
+        test_all_testaddons(env_root, defaults=options.__dict__)
+        return
     elif command == "testex":
         test_all_examples(env_root, defaults=options.__dict__)
         return
@@ -517,19 +574,26 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
         test_all(env_root, defaults=options.__dict__)
         return
     elif command == "testcfx":
+        if options.filter:
+            print >>sys.stderr, "The filter option is not valid with the testcfx command"
+            return
         test_cfx(env_root, options.verbose)
         return
     elif command == "docs":
         from cuddlefish.docs import generate
         if len(args) > 1:
-            docs_home = generate.generate_named_file(env_root, filename=args[1])
+            docs_home = generate.generate_named_file(env_root, filename_and_path=args[1])
         else:
             docs_home = generate.generate_local_docs(env_root)
             webbrowser.open(docs_home)
         return
     elif command == "sdocs":
         from cuddlefish.docs import generate
-        filename = generate.generate_static_docs(env_root)
+        filename=""
+        if options.override_version:
+            filename = generate.generate_static_docs(env_root, override_version=options.override_version)
+        else:
+            filename = generate.generate_static_docs(env_root)
         print >>stdout, "Wrote %s." % filename
         return
     elif command not in ["xpi", "test", "run"]:
@@ -625,7 +689,8 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
 
     deps = packaging.get_deps_for_targets(pkg_cfg, targets)
 
-    from cuddlefish.manifest import build_manifest, ModuleNotFoundError
+    from cuddlefish.manifest import build_manifest, ModuleNotFoundError, \
+                                    BadChromeMarkerError
     # Figure out what loader files should be scanned. This is normally
     # computed inside packaging.generate_build_for_target(), by the first
     # dependent package that defines a "loader" property in its package.json.
@@ -638,11 +703,11 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
     # the choice of loader for manifest-generation purposes. In practice,
     # this means that alternative loaders probably won't work with
     # --strip-xpi.
-    assert packaging.DEFAULT_LOADER == "api-utils"
-    assert pkg_cfg.packages["api-utils"].loader == "lib/cuddlefish.js"
-    cuddlefish_js_path = os.path.join(pkg_cfg.packages["api-utils"].root_dir,
-                                      "lib", "cuddlefish.js")
-    loader_modules = [("api-utils", "lib", "cuddlefish", cuddlefish_js_path)]
+    assert packaging.DEFAULT_LOADER == "addon-sdk"
+    assert pkg_cfg.packages["addon-sdk"].loader == "lib/sdk/loader/cuddlefish.js"
+    cuddlefish_js_path = os.path.join(pkg_cfg.packages["addon-sdk"].root_dir,
+                                      "lib", "sdk", "loader", "cuddlefish.js")
+    loader_modules = [("addon-sdk", "lib", "sdk/loader/cuddlefish", cuddlefish_js_path)]
     scan_tests = command == "test"
     test_filter_re = None
     if scan_tests and options.filter:
@@ -656,6 +721,9 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
     except ModuleNotFoundError, e:
         print str(e)
         sys.exit(1)
+    except BadChromeMarkerError, e:
+        # An error had already been displayed on stderr in manifest code
+        sys.exit(1)
     used_deps = manifest.get_used_packages()
     if command == "test":
         # The test runner doesn't appear to link against any actual packages,
@@ -685,8 +753,8 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
     if command == "test":
         # This should be contained in the test runner package.
         # maybe just do: target_cfg.main = 'test-harness/run-tests'
-        harness_options['main'] = 'test-harness/run-tests'
-        harness_options['mainPath'] = manifest.get_manifest_entry("test-harness", "lib", "run-tests").get_path()
+        harness_options['main'] = 'sdk/test/runner'
+        harness_options['mainPath'] = manifest.get_manifest_entry("addon-sdk", "lib", "sdk/test/runner").get_path()
     else:
         harness_options['main'] = target_cfg.get('main')
         harness_options['mainPath'] = manifest.top_path
@@ -707,7 +775,7 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
         app_extension_dir = os.path.abspath(options.templatedir)
     else:
         mydir = os.path.dirname(os.path.abspath(__file__))
-        app_extension_dir = os.path.join(mydir, "app-extension")
+        app_extension_dir = os.path.join(mydir, "../../app-extension")
 
 
     if target_cfg.get('preferences'):
@@ -715,6 +783,8 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
 
     harness_options['manifest'] = manifest.get_harness_options_manifest()
     harness_options['allTestModules'] = manifest.get_all_test_modules()
+    if len(harness_options['allTestModules']) == 0 and command == "test":
+        sys.exit(0)
 
     from cuddlefish.rdf import gen_manifest, RDFUpdate
 
@@ -726,6 +796,8 @@ def run(arguments=sys.argv[1:], target_cfg=None, pkg_cfg=None,
                                 enable_mobile=options.enable_mobile)
 
     if command == "xpi" and options.update_link:
+        if not options.update_link.startswith("https"):
+            raise optparse.OptionValueError("--update-link must start with 'https': %s" % options.update_link)
         rdf_name = UPDATE_RDF_FILENAME % target_cfg.name
         print >>stdout, "Exporting update description to %s." % rdf_name
         update = RDFUpdate()
