@@ -1,7 +1,17 @@
+# This Source Code Form is subject to the terms of the Mozilla Public
+# License, v. 2.0. If a copy of the MPL was not distributed with this
+# file, You can obtain one at http://mozilla.org/MPL/2.0/.
+
 
 import os, sys, re, hashlib
 import simplejson as json
 SEP = os.path.sep
+from cuddlefish.util import filter_filenames, filter_dirnames
+
+# Load new layout mapping hashtable
+path = os.path.join(os.environ.get('CUDDLEFISH_ROOT'), "mapping.json")
+data = open(path, 'r').read()
+NEW_LAYOUT_MAPPING = json.loads(data)
 
 def js_zipname(packagename, modulename):
     return "%s-lib/%s.js" % (packagename, modulename)
@@ -16,17 +26,22 @@ def to_json(o):
     return json.dumps(o, indent=1).encode("utf-8")+"\n"
 
 class ModuleNotFoundError(Exception):
-    def __init__(self, *args):
-        self.args = args
-        self.used_by = None # string, full path to module which did require()
-        self.requirement_name = None # string, what they require()d
-        self.looked_at = [] # list of full paths to potential .js files
+    def __init__(self, requirement_type, requirement_name,
+                 used_by, line_number, looked_in):
+        Exception.__init__(self)
+        self.requirement_type = requirement_type # "require" or "define"
+        self.requirement_name = requirement_name # string, what they require()d
+        self.used_by = used_by # string, full path to module which did require()
+        self.line_number = line_number # int, 1-indexed line number of first require()
+        self.looked_in = looked_in # list of full paths to potential .js files
     def __str__(self):
-        return ("ModuleNotFoundError: unable to satisfy require(%s) from %s .\n"
-                "Looked for it in:\n"
-                " %s\n" %
-                (self.requirement_name, self.used_by,
-                 "\n ".join(self.looked_in)))
+        what = "%s(%s)" % (self.requirement_type, self.requirement_name)
+        where = self.used_by
+        if self.line_number is not None:
+            where = "%s:%d" % (self.used_by, self.line_number)
+        searched = "Looked for it in:\n  %s\n" % "\n  ".join(self.looked_in)
+        return ("ModuleNotFoundError: unable to satisfy: %s from\n"
+                "  %s:\n" % (what, where)) + searched
 
 class BadModuleIdentifier(Exception):
     pass
@@ -42,11 +57,14 @@ class ManifestEntry:
         self.requirements = {}
         self.datamap = None
 
-    def get_uri(self, prefix):
-        return "%s%s-%s/%s.js" % \
-               (prefix, self.packageName, self.sectionName, self.moduleName)
+    def get_path(self):
+        path = "%s/%s/%s" % \
+               (self.packageName, self.sectionName, self.moduleName)
+        if not path.endswith(".js"):
+          path += ".js"
+        return path
 
-    def get_entry_for_manifest(self, prefix):
+    def get_entry_for_manifest(self):
         entry = { "packageName": self.packageName,
                   "sectionName": self.sectionName,
                   "moduleName": self.moduleName,
@@ -55,20 +73,15 @@ class ManifestEntry:
                   "requirements": {},
                   }
         for req in self.requirements:
-            if self.requirements[req]:
+            if isinstance(self.requirements[req], ManifestEntry):
                 them = self.requirements[req] # this is another ManifestEntry
-                them_uri = them.get_uri(prefix)
-                entry["requirements"][req] = {"uri": them_uri}
+                them_path = them.get_path()
+                entry["requirements"][req] = {"path": them_path}
             else:
                 # something magic. The manifest entry indicates that they're
                 # allowed to require() it
-                entry["requirements"][req] = {}
-        if self.datamap:
-            entry["requirements"]["self"] = {
-                "mapSHA256": self.datamap.data_manifest_hash,
-                "mapName": self.packageName+"-data",
-                "dataURIPrefix": "%s%s-data/" % (prefix, self.packageName),
-                }
+                entry["requirements"][req] = self.requirements[req]
+            assert isinstance(entry["requirements"][req], dict)
         return entry
 
     def add_js(self, js_filename):
@@ -95,27 +108,12 @@ class ManifestEntry:
 def hash_file(fn):
     return hashlib.sha256(open(fn,"rb").read()).hexdigest()
 
-# things to ignore in data/ directories
-IGNORED_FILES = [".hgignore"]
-IGNORED_FILE_SUFFIXES = ["~"]
-IGNORED_DIRS = [".svn", ".hg", "defaults"]
-
-def filter_filenames(filenames):
-    for filename in filenames:
-        if filename in IGNORED_FILES:
-            continue
-        if any([filename.endswith(suffix)
-                for suffix in IGNORED_FILE_SUFFIXES]):
-            continue
-        yield filename
-
 def get_datafiles(datadir):
     # yields pathnames relative to DATADIR, ignoring some files
     for dirpath, dirnames, filenames in os.walk(datadir):
         filenames = list(filter_filenames(filenames))
         # this tells os.walk to prune the search
-        dirnames[:] = [dirname for dirname in dirnames
-                       if dirname not in IGNORED_DIRS]
+        dirnames[:] = filter_dirnames(dirnames)
         for filename in filenames:
             fullname = os.path.join(dirpath, filename)
             assert fullname.startswith(datadir+SEP), "%s%s not in %s" % (datadir, SEP, fullname)
@@ -124,7 +122,7 @@ def get_datafiles(datadir):
 
 class DataMap:
     # one per package
-    def __init__(self, pkg, uri_prefix):
+    def __init__(self, pkg):
         self.pkg = pkg
         self.name = pkg.name
         self.files_to_copy = []
@@ -138,7 +136,7 @@ class DataMap:
         self.data_manifest = to_json(datamap)
         self.data_manifest_hash = hashlib.sha256(self.data_manifest).hexdigest()
         self.data_manifest_zipname = datamap_zipname(pkg.name)
-        self.data_uri_prefix = "%s%s-data/" % (uri_prefix, self.name)
+        self.data_uri_prefix = "%s/data/" % (self.name)
 
 class BadChromeMarkerError(Exception):
     pass
@@ -169,7 +167,7 @@ class ModuleInfo:
                                                    self.js, self.docs)
 
 class ManifestBuilder:
-    def __init__(self, target_cfg, pkg_cfg, deps, uri_prefix, extra_modules,
+    def __init__(self, target_cfg, pkg_cfg, deps, extra_modules,
                  stderr=sys.stderr):
         self.manifest = {} # maps (package,section,module) to ManifestEntry
         self.target_cfg = target_cfg # the entry point
@@ -177,39 +175,66 @@ class ManifestBuilder:
         self.deps = deps # list of package names to search
         self.used_packagenames = set()
         self.stderr = stderr
-        self.uri_prefix = uri_prefix
         self.extra_modules = extra_modules
         self.modules = {} # maps ModuleInfo to URI in self.manifest
         self.datamaps = {} # maps package name to DataMap instance
         self.files = [] # maps manifest index to (absfn,absfn) js/docs pair
+        self.test_modules = [] # for runtime
 
-    def build(self, scan_tests):
+    def build(self, scan_tests, test_filter_re):
         # process the top module, which recurses to process everything it
         # reaches
         if "main" in self.target_cfg:
-            self.top_uri = self.process_module(self.find_top(self.target_cfg))
+            top_mi = self.find_top(self.target_cfg)
+            top_me = self.process_module(top_mi)
+            self.top_path = top_me.get_path()
+            self.datamaps[self.target_cfg.name] = DataMap(self.target_cfg)
         if scan_tests:
-            # also scan all test files in all packages that we use
-            for packagename in self.used_packagenames:
-                package = self.pkg_cfg.packages[packagename]
-                dirnames = package["tests"]
-                if isinstance(dirnames, basestring):
-                    dirnames = [dirnames]
-                dirnames = [os.path.join(package.root_dir, d) for d in dirnames]
-                for d in dirnames:
-                    for tname in os.listdir(d):
-                        if tname.startswith("test-") and tname.endswith(".js"):
-                            #re.search(r'^test-.*\.js$', tname):
-                            tmi = ModuleInfo(package, "tests", tname[:-3],
-                                             os.path.join(d, tname), None)
-                            self.process_module(tmi)
+            mi = self._find_module_in_package("addon-sdk", "lib", "sdk/test/runner", [])
+            self.process_module(mi)
+            # also scan all test files in all packages that we use. By making
+            # a copy of self.used_packagenames first, we refrain from
+            # processing tests in packages that our own tests depend upon. If
+            # we're running tests for package A, and either modules in A or
+            # tests in A depend upon modules from package B, we *don't* want
+            # to run tests for package B.
+            test_modules = []
+            dirnames = self.target_cfg["tests"]
+            if isinstance(dirnames, basestring):
+                dirnames = [dirnames]
+            dirnames = [os.path.join(self.target_cfg.root_dir, d)
+                        for d in dirnames]
+            for d in dirnames:
+                for filename in os.listdir(d):
+                    if filename.startswith("test-") and filename.endswith(".js"):
+                        testname = filename[:-3] # require(testname)
+                        if test_filter_re:
+                            if not re.search(test_filter_re, testname):
+                                continue
+                        tmi = ModuleInfo(self.target_cfg, "tests", testname,
+                                         os.path.join(d, filename), None)
+                        # scan the test's dependencies
+                        tme = self.process_module(tmi)
+                        test_modules.append( (testname, tme) )
+            # also add it as an artificial dependency of unit-test-finder, so
+            # the runtime dynamic load can work.
+            test_finder = self.get_manifest_entry("addon-sdk", "lib",
+                                                  "sdk/deprecated/unit-test-finder")
+            for (testname,tme) in test_modules:
+                test_finder.add_requirement(testname, tme)
+                # finally, tell the runtime about it, so they won't have to
+                # search for all tests. self.test_modules will be passed
+                # through the harness-options.json file in the
+                # .allTestModules property.
+                self.test_modules.append(testname)
+
         # include files used by the loader
         for em in self.extra_modules:
             (pkgname, section, modname, js) = em
             mi = ModuleInfo(self.pkg_cfg.packages[pkgname], section, modname,
                             js, None)
             self.process_module(mi)
-        
+
 
     def get_module_entries(self):
         return frozenset(self.manifest.values())
@@ -227,17 +252,21 @@ class ManifestBuilder:
         # returns all .js files that we reference, plus data/ files. You will
         # need to add the loader, off-manifest files that it needs, and
         # generated metadata.
+        for datamap in self.datamaps.values():
+            for (zipname, absname) in datamap.files_to_copy:
+                yield absname
+
         for me in self.get_module_entries():
             yield me.js_filename
-            if me.datamap:
-                for (zipname, absname) in me.datamap.files_to_copy:
-                    yield absname
 
-    def get_harness_options_manifest(self, uri_prefix):
+    def get_all_test_modules(self):
+        return self.test_modules
+
+    def get_harness_options_manifest(self):
         manifest = {}
         for me in self.get_module_entries():
-            uri = me.get_uri(uri_prefix)
-            manifest[uri] = me.get_entry_for_manifest(uri_prefix)
+            path = me.get_path()
+            manifest[path] = me.get_entry_for_manifest()
         return manifest
 
     def get_manifest_entry(self, package, section, module):
@@ -337,7 +366,7 @@ class ManifestBuilder:
             me.add_docs(mi.docs)
 
         js_lines = open(mi.js,"r").readlines()
-        requires, problems = scan_module(mi.js, js_lines, self.stderr)
+        requires, problems, locations = scan_module(mi.js,js_lines,self.stderr)
         if problems:
             # the relevant instructions have already been written to stderr
             raise BadChromeMarkerError()
@@ -346,16 +375,10 @@ class ManifestBuilder:
         # traversal of the module graph
 
         for reqname in sorted(requires.keys()):
-            if reqname in ("chrome", "parent-loader", "loader", "manifest"):
-                me.add_requirement(reqname, None)
-            elif reqname == "self":
-                # this might reference bundled data, so:
-                #  1: hash that data, add the hash as a dependency
-                #  2: arrange for the data to be copied into the XPI later
-                if pkg.name not in self.datamaps:
-                    self.datamaps[pkg.name] = DataMap(pkg, self.uri_prefix)
-                dm = self.datamaps[pkg.name]
-                me.add_data(dm) # 'self' is implicit
+            # If requirement is chrome or a pseudo-module (starts with @) make
+            # path a requirement name.
+            if reqname == "chrome" or reqname.startswith("@"):
+                me.add_requirement(reqname, {"path": reqname})
             else:
                 # when two modules require() the same name, do they get a
                 # shared instance? This is a deep question. For now say yes.
@@ -368,12 +391,19 @@ class ManifestBuilder:
                 looked_in = [] # populated by subroutines
                 them_me = self.find_req_for(mi, reqname, looked_in)
                 if them_me is None:
-                    err = ModuleNotFoundError()
-                    err.used_by = mi.js
-                    err.requirement_name = reqname
-                    err.looked_in = looked_in
+                    if mi.section == "tests":
+                        # tolerate missing modules in tests, because
+                        # test-securable-module.js, and the modules/red.js
+                        # that it imports, both do that intentionally
+                        continue
+                    lineno = locations.get(reqname) # None means define()
+                    if lineno is None:
+                        reqtype = "define"
+                    else:
+                        reqtype = "require"
+                    err = ModuleNotFoundError(reqtype, reqname,
+                                              mi.js, lineno, looked_in)
                     raise err
-                    #print "Warning: unable to satisfy require(%s) from %s" % (reqname, mi)
                 else:
                     me.add_requirement(reqname, them_me)
 
@@ -402,12 +432,12 @@ class ManifestBuilder:
         modulename = from_module.name
 
         #print " %s require(%s))" % (from_module, reqname)
-        bits = reqname.split("/")
 
         if reqname.startswith("./") or reqname.startswith("../"):
             # 1: they want something relative to themselves, always from
             # their own package
             them = modulename.split("/")[:-1]
+            bits = reqname.split("/")
             while bits[0] in (".", ".."):
                 if not bits:
                     raise BAD("no actual modulename")
@@ -426,8 +456,21 @@ class ManifestBuilder:
         # non-relative import. Might be a short name (requiring a search
         # through "library" packages), or a fully-qualified one.
 
+        # Search for a module in new layout.
+        # First normalize require argument in order to easily find a mapping
+        normalized = reqname
+        if normalized.endswith(".js"):
+            normalized = normalized[:-len(".js")]
+        if normalized.startswith("addon-kit/"):
+            normalized = normalized[len("addon-kit/"):]
+        if normalized.startswith("api-utils/"):
+            normalized = normalized[len("api-utils/"):]
+        if normalized in NEW_LAYOUT_MAPPING:
+          reqname = NEW_LAYOUT_MAPPING[normalized]
+
         if "/" in reqname:
             # 2: PKG/MOD: find PKG, look inside for MOD
+            bits = reqname.split("/")
             lookfor_pkg = bits[0]
             lookfor_mod = "/".join(bits[1:])
             mi = self._get_module_from_package(lookfor_pkg,
@@ -519,24 +562,30 @@ class ManifestBuilder:
     def _find_module_in_package(self, pkgname, sections, name, looked_in):
         # require("a/b/c") should look at ...\a\b\c.js on windows
         filename = os.sep.join(name.split("/"))
+        # normalize filename, make sure that we do not add .js if it already has
+        # it.
+        if not filename.endswith(".js"):
+          filename += ".js"
+        basename = filename[:-3]
+
         pkg = self.pkg_cfg.packages[pkgname]
         if isinstance(sections, basestring):
             sections = [sections]
         for section in sections:
             for sdir in pkg.get(section, []):
-                js = os.path.join(pkg.root_dir, sdir, filename+".js")
+                js = os.path.join(pkg.root_dir, sdir, filename)
                 looked_in.append(js)
                 if os.path.exists(js):
                     docs = None
                     maybe_docs = os.path.join(pkg.root_dir, "docs",
-                                              filename+".md")
+                                              basename+".md")
                     if section == "lib" and os.path.exists(maybe_docs):
                         docs = maybe_docs
                     return ModuleInfo(pkg, section, name, js, docs)
         return None
 
-def build_manifest(target_cfg, pkg_cfg, deps, uri_prefix, scan_tests,
-                   extra_modules=[]):
+def build_manifest(target_cfg, pkg_cfg, deps, scan_tests,
+                   test_filter_re=None, extra_modules=[]):
     """
     Perform recursive dependency analysis starting from entry_point,
     building up a manifest of modules that need to be included in the XPI.
@@ -562,13 +611,13 @@ def build_manifest(target_cfg, pkg_cfg, deps, uri_prefix, scan_tests,
     code which does, so it knows what to copy into the XPI.
     """
 
-    mxt = ManifestBuilder(target_cfg, pkg_cfg, deps, uri_prefix, extra_modules)
-    mxt.build(scan_tests)
+    mxt = ManifestBuilder(target_cfg, pkg_cfg, deps, extra_modules)
+    mxt.build(scan_tests, test_filter_re)
     return mxt
 
 
 
-COMMENT_PREFIXES = ["//", "/*", "*", "\'", "\"", "dump("]
+COMMENT_PREFIXES = ["//", "/*", "*", "dump("]
 
 REQUIRE_RE = r"(?<![\'\"])require\s*\(\s*[\'\"]([^\'\"]+?)[\'\"]\s*\)"
 
@@ -582,7 +631,8 @@ DEF_RE_ALLOWED = re.compile(r"^[\'\"][^\'\"]+[\'\"]$")
 
 def scan_requirements_with_grep(fn, lines):
     requires = {}
-    for line in lines:
+    first_location = {}
+    for (lineno0, line) in enumerate(lines):
         for clause in line.split(";"):
             clause = clause.strip()
             iscomment = False
@@ -595,6 +645,8 @@ def scan_requirements_with_grep(fn, lines):
             if mo:
                 modname = mo.group(1)
                 requires[modname] = {}
+                if modname not in first_location:
+                    first_location[modname] = lineno0+1
 
     # define() can happen across multiple lines, so join everyone up.
     wholeshebang = "\n".join(lines)
@@ -610,8 +662,10 @@ def scan_requirements_with_grep(fn, lines):
                 modname = strbit[1:-1]
                 if modname not in ["exports"]:
                     requires[modname] = {}
+                    # joining all the lines means we lose line numbers, so we
+                    # can't fill first_location[]
 
-    return requires
+    return requires, first_location
 
 CHROME_ALIASES = [
     (re.compile(r"Components\.classes"), "Cc"),
@@ -625,6 +679,7 @@ OTHER_CHROME = re.compile(r"Components\.[a-zA-Z]")
 def scan_for_bad_chrome(fn, lines, stderr):
     problems = False
     old_chrome = set() # i.e. "Cc" when we see "Components.classes"
+    old_chrome_lines = [] # list of (lineno, line.strip()) tuples
     for lineno,line in enumerate(lines):
         # note: this scanner is not obligated to spot all possible forms of
         # chrome access. The scanner is detecting voluntary requests for
@@ -645,35 +700,49 @@ def scan_for_bad_chrome(fn, lines, stderr):
             if OTHER_CHROME.search(line):
                 old_chrome_in_this_line.add("components")
         old_chrome.update(old_chrome_in_this_line)
-                
+        if old_chrome_in_this_line:
+            old_chrome_lines.append( (lineno+1, line) )
+
     if old_chrome:
-        print >>stderr, ""
-        print >>stderr, "To use chrome authority, you need a line like this:"
-        needs = ",".join(sorted(old_chrome))
-        print >>stderr, '  const {%s} = require("chrome");' % needs
-        print >>stderr, "because things like 'Components.classes' will not be available"
+        print >>stderr, """
+The following lines from file %(fn)s:
+%(lines)s
+use 'Components' to access chrome authority. To do so, you need to add a
+line somewhat like the following:
+
+  const {%(needs)s} = require("chrome");
+
+Then you can use any shortcuts to its properties that you import from the
+'chrome' module ('Cc', 'Ci', 'Cm', 'Cr', and 'Cu' for the 'classes',
+'interfaces', 'manager', 'results', and 'utils' properties, respectively. And
+`components` for `Components` object itself).
+""" % { "fn": fn, "needs": ",".join(sorted(old_chrome)),
+        "lines": "\n".join([" %3d: %s" % (lineno,line)
+                            for (lineno, line) in old_chrome_lines]),
+        }
         problems = True
     return problems
 
 def scan_module(fn, lines, stderr=sys.stderr):
     filename = os.path.basename(fn)
-    requires = scan_requirements_with_grep(fn, lines)
-    if filename == "cuddlefish.js" or filename == "securable-module.js":
-        # these are the loader: don't scan for chrome
+    requires, locations = scan_requirements_with_grep(fn, lines)
+    if filename == "cuddlefish.js":
+        # this is the loader: don't scan for chrome
         problems = False
     else:
         problems = scan_for_bad_chrome(fn, lines, stderr)
-    return requires, problems
+    return requires, problems, locations
 
 
 
 if __name__ == '__main__':
     for fn in sys.argv[1:]:
-        requires,problems = scan_module(fn, open(fn).readlines())
+        requires, problems, locations = scan_module(fn, open(fn).readlines())
         print
         print "---", fn
         if problems:
             print "PROBLEMS"
             sys.exit(1)
-        print "requires: %s" % (",".join(requires))
+        print "requires: %s" % (",".join(sorted(requires.keys())))
+        print "locations: %s" % locations
 
